/*
 * configdialog.cpp
 *
 * configuration dialog for xmms-kde
 */
#include <config.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qfileinfo.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qcolordialog.h>


#include <kaccel.h>
#include <kkeydialog.h>
#include <kglobalaccel.h>
#include <kdialogbase.h>
#include <kconfig.h>
#include <klocale.h>
#include <ksimpleconfig.h>
#include <ktar.h>
#include <kstddirs.h>
#include <kmimetype.h>
#include <kfilemetainfo.h>

#include "osdframe.h"
//#include "xmms-kdedb.h"

#include "configdialog.h"
#include "configdialog.moc"




XmmsKdeConfigDialog::XmmsKdeConfigDialog(QWidget *parent, 
					 const QStringList themeList, 
					 KStandardDirs *dataDirs,
                                         int currentScrollMode, 
					 KGlobalAccel *globalAccel,
					 QString currentTheme,
					 PlayerInterface::Player currentPlayer,
					 bool minXmmsPlayer,
					 bool minXmmsPlaylist,
					 bool readTags,
					 bool underscores,
					 bool enableOsd, int osdSize,
					 int osdx, int osdy,
					 QColor osdFGColor, QColor osdBGColor,
					 float osdTransparency
					 /*XmmsKdeDB *datab*/)  
  : KDialogBase (IconList, i18n("Options"), Ok|Cancel, Ok, parent, "configdialog", true, true) {

  scrollMode  = currentScrollMode;
  tlist       = themeList;
  accel       = globalAccel;

  nameList = new QStringList();

  for (unsigned int index = 0; index < tlist.count(); index ++) 
    nameList->append(QFileInfo(tlist[index]).fileName());

  dDirs = dataDirs;

  noScreen = new QPixmap(readFile(dDirs->findResource("data", "xmms-kde/noscreen.tgz"), "noscreen.png"));

  osd = 0;
  //db = datab;


  setupThemesPage(currentTheme);
  setupPlayerPage(currentPlayer, minXmmsPlayer, minXmmsPlaylist, readTags,
		  underscores);
  setupAccelPage();
  setupOSDPage(enableOsd, osdSize, osdx, osdy, osdFGColor, osdBGColor,
	       osdTransparency);
  //setupDBPage(db);

  setFixedSize(400, 280);

  selectedPlayer = currentPlayer;
}




XmmsKdeConfigDialog::~XmmsKdeConfigDialog() {

  if (themeListBox) delete themeListBox;
  if (imageLabel) delete imageLabel;
  if (nameList) delete nameList;
  if (noScreen) delete noScreen;

  if (osd) delete osd;
}


/*
 * setup the Theme configuration page
 */
void XmmsKdeConfigDialog::setupThemesPage(QString currentTheme) {
  
  QPixmap themes(readFile(dDirs->findResource("data", "xmms-kde/icons.tgz"), "themes.png"));

  QWidget *page = addPage(i18n("Themes"), "Themes", themes);
  
  QVBoxLayout *vbox = new QVBoxLayout(page);
  QHBox *box = new QHBox(page);
  vbox->addWidget(box);
  
  themeListBox = new QListBox(box);
  themeListBox->insertStringList(*nameList);
  themeListBox->setFixedSize(200, 120);

  connect(themeListBox, SIGNAL(highlighted(int)), this, SLOT(showTheme(int)));

  imageLabel = new QLabel(box);

  imageLabel->setFixedSize(140, 100);
  imageLabel->setAlignment(Qt::AlignCenter);

  // set scroll mode
  QButtonGroup *scrollGroup = new QButtonGroup(3, Qt::Horizontal, page);
  vbox->addWidget(scrollGroup);
  QRadioButton *btn1 = new QRadioButton("Loop", scrollGroup); // 0
  QRadioButton *btn2 = new QRadioButton("Ping pong", scrollGroup); // 1
  QRadioButton *btn3 = new QRadioButton("Sinus", scrollGroup); // 2
  scrollGroup->setButton(scrollMode);
  scrollGroup->setTitle("Title scroll mode");
  connect(scrollGroup, SIGNAL(clicked(int)), this, 
	  SLOT(scrollModeChanged(int)));
  
  themeListBox->setCurrentItem(themeListBox->findItem(QFileInfo(currentTheme).fileName()));  

  
}

void XmmsKdeConfigDialog::setupAccelPage() {

  QPixmap shortcuts(readFile(dDirs->findResource("data", "xmms-kde/icons.tgz"), "shortcuts.png"));

  QWidget *page = addPage(i18n("Shortcuts"), "Shortcuts", shortcuts);
  
  QVBoxLayout *vbox = new QVBoxLayout(page);
  accelBox = new QCheckBox(i18n("enable Shortcuts"), page);
  vbox->addWidget(accelBox);

  QPushButton *accelButton = new QPushButton("Change Shortcuts", page);
  vbox->addWidget(accelButton);

  connect(accelButton, SIGNAL(clicked()), this, SLOT(accelButtonClicked()));

  if (accel->isEnabled()) accelBox->setChecked(true);
  //else accelBox->setChecked(false);
}

void XmmsKdeConfigDialog::setupOSDPage(bool enable, int size, int x, int y,
				       QColor osdFGColor, QColor osdBGColor,
				       float osdTransparency) {

  QPixmap osd(readFile(dDirs->findResource("data", "xmms-kde/icons.tgz"), "osd.png"));

  QWidget *page = addPage(i18n("OSD"), "On Screen Display", osd);
  
  QVBoxLayout *vbox = new QVBoxLayout(page);
  osdBox = new QCheckBox(i18n("enable On Screen Display"), page);
  osdBox->setChecked(enable);
  vbox->addWidget(osdBox);

  QHBox *box = new QHBox(page);
  QLabel *osdSize = new QLabel("Font size: ", box);
  osdSizeBox = new QSpinBox(10, 100, 1, box);
  osdSizeBox->setValue(size);
  vbox->addWidget(box);

  box = new QHBox(page);
  QLabel *xpos = new QLabel("x Position: ", box);
  osdXBox = new QSpinBox(0, 1600, 1, box);
  osdXBox->setValue(x);
  vbox->addWidget(box);

  box = new QHBox(page);
  QLabel *ypos = new QLabel("y Position: ", box);
  osdYBox = new QSpinBox(0, 1200, 1, box);
  osdYBox->setValue(y);
  vbox->addWidget(box);

  box = new QHBox(page);
  QLabel *fgcolor = new QLabel("Text Color: ", box);
  osdFGColorButton = new KColorButton(osdFGColor, box);
  vbox->addWidget(box);

  box = new QHBox(page);
  QLabel *bgcolor = new QLabel("Background Color: ", box);
  osdBGColorButton = new KColorButton(osdBGColor, box);
  vbox->addWidget(box);

  box = new QHBox(page);
  QLabel *transparency = new QLabel("Transparency: ", box);
  osdTBox = new QSpinBox(0, 100, 1, box);
  osdTBox->setValue(100 - osdTransparency * 100);
  vbox->addWidget(box);

  connect(osdXBox, SIGNAL(valueChanged(int)), this, SLOT(osdChanged()));
  connect(osdYBox, SIGNAL(valueChanged(int)), this, SLOT(osdChanged()));
  connect(osdSizeBox, SIGNAL(valueChanged(int)), this, SLOT(osdChanged()));
  connect(osdTBox, SIGNAL(valueChanged(int)), this, SLOT(osdChanged()));

  connect(osdFGColorButton, SIGNAL(clicked()), this, SLOT(osdChanged()));
  connect(osdBGColorButton, SIGNAL(clicked()), this, SLOT(osdChanged()));
}

/*
void XmmsKdeConfigDialog::setupDBPage(XmmsKdeDB *db) {

  QPixmap dbPix(readFile(dDirs->findResource("data", "xmms-kde/icons.tgz"), "db.png"));

  QWidget *page = addPage(i18n("Database"), "Database", dbPix);
  QVBoxLayout *pagebox = new QVBoxLayout(page);

  QTabWidget *confWidget = new QTabWidget(page);
  QWidget *dbWidget = new QWidget(confWidget);  
  
  QVBoxLayout *vbox = new QVBoxLayout(dbWidget);

  QHBox *box = new QHBox(dbWidget);
  dbBox = new QCheckBox(i18n("enable Database support"), box);
  dbBox->setChecked(db->isEnabled());
  QPushButton *b = new QPushButton("test", box);
  connect(b, SIGNAL(clicked()), this, SLOT(testDB()));  
  vbox->addWidget(box);

  box = new QHBox(dbWidget);
  QLabel *l = new QLabel("Hostname: ", box);
  dbHostEdit = new QLineEdit(box);
  dbHostEdit->setText(db->getHost());
  vbox->addWidget(box);

  box = new QHBox(dbWidget);
  l = new QLabel("Port: ", box);
  dbPortEdit = new QLineEdit(box);
  dbPortEdit->setText(QString().setNum(db->getPort()));
  vbox->addWidget(box);
  
  box = new QHBox(dbWidget);
  l = new QLabel("Database name: ", box);
  dbNameEdit = new QLineEdit(box);
  dbNameEdit->setText(db->getName());
  vbox->addWidget(box);
  
  box = new QHBox(dbWidget);
  l = new QLabel("Database user: ", box);
  dbUserEdit = new QLineEdit(box);
  dbUserEdit->setText(db->getUser());
  vbox->addWidget(box);

  box = new QHBox(dbWidget);
  l = new QLabel("User password: ", box);
  dbPassEdit = new QLineEdit(box);
  dbPassEdit->setText(db->getPass());
  vbox->addWidget(box);

  // file tab
  QWidget *fileWidget = new QWidget(confWidget);    
  vbox = new QVBoxLayout(fileWidget);

  box = new QHBox(fileWidget);
  l = new QLabel("Path: ", box);
  pathEdit = new QLineEdit(box);
  QPushButton *browseButton = new QPushButton("browse", box);
  QPushButton *addButton = new QPushButton("add", box);
  vbox->addWidget(box);
  connect(browseButton, SIGNAL(clicked()), this, SLOT(setCurrentPath()));
  connect(addButton, SIGNAL(clicked()), this, SLOT(addCurrentPath()));

  pathList = new QListBox(fileWidget);
  vbox->addWidget(pathList);

  QPushButton *dbAddButton = new QPushButton("Insert into Database", fileWidget);
  vbox->addWidget(dbAddButton, 0, Qt::AlignRight);
  connect(dbAddButton, SIGNAL(clicked()), this, SLOT(addSelectionToDatabase()));
  
  confWidget->addTab(dbWidget, "Database");
  confWidget->addTab(fileWidget, "Files");

  pagebox->addWidget(confWidget);

  dbStatusLabel = new QLabel("", page);
  pagebox->addWidget(dbStatusLabel);
  connect(db, SIGNAL(statusChanged(QString)), 
	  this, SLOT(setDBStatus(QString)));

}
*/

void XmmsKdeConfigDialog::setupPlayerPage(PlayerInterface::Player currentPlayer,
					  bool minXmmsPlayer,
					  bool minXmmsPlaylist,
					  bool readTags,
					  bool underscores) {

  QPixmap playerIcon(readFile(dDirs->findResource("data", "xmms-kde/icons.tgz"), "players.png"));
  QPixmap okIcon(readFile(dDirs->findResource("data", "xmms-kde/icons.tgz"), "ok.png"));
  QPixmap koIcon(readFile(dDirs->findResource("data", "xmms-kde/icons.tgz"), "ko.png"));

  QWidget *page = addPage(i18n("Player"), "Player", playerIcon);
  
  QVBoxLayout *vbox = new QVBoxLayout(page);
  vbox->setSpacing(10);

  playerGroup = new QButtonGroup(3, Qt::Horizontal, page);

  QRadioButton *xmmsButton = new QRadioButton("XMMS", playerGroup);
  QRadioButton *noatunButton = new QRadioButton("Noatun", playerGroup);
  QRadioButton *smpegButton = new QRadioButton("SMPEG", playerGroup);

  playerGroup->setButton(currentPlayer);
  playerGroup->setTitle("Player");

  connect(playerGroup, SIGNAL(clicked(int)), 
	  this, SLOT(playerChanged(int)));

  /*
   * build the xmms configuration widget
   */
  QTabWidget *confWidget = new QTabWidget(page);
  
  QWidget *xmmsWidget = new QWidget(confWidget);  
  QVBoxLayout *xvbox = new QVBoxLayout(xmmsWidget);
  minimizeXmmsPlayer = new QCheckBox("don't show player at startup",
				     xmmsWidget);
  minimizeXmmsPlaylist = new QCheckBox("don't show playlist at startup",
				       xmmsWidget);
  xvbox->addWidget(minimizeXmmsPlayer);
  xvbox->addWidget(minimizeXmmsPlaylist);

  minimizeXmmsPlayer->setChecked(minXmmsPlayer);
  minimizeXmmsPlaylist->setChecked(minXmmsPlaylist);

  confWidget->addTab(xmmsWidget, "XMMS");

  /*
   * build the noatun configuration widget
   */
  QWidget *noatunWidget = new QWidget(confWidget);  
  xvbox = new QVBoxLayout(noatunWidget);

  confWidget->addTab(noatunWidget, "Noatun");

  /*
   * build the smpeg configuration widget
   */
  QWidget *smpegWidget = new QWidget(confWidget);  
  xvbox = new QVBoxLayout(smpegWidget);
  readID3Tags = new QCheckBox("read ID3 tags when files are dropped",
			      smpegWidget);
  xvbox->addWidget(readID3Tags);
  readID3Tags->setChecked(readTags);

  convertUnderscores = new QCheckBox("convert underscores to spaces",
				     smpegWidget);
  xvbox->addWidget(convertUnderscores);
  convertUnderscores->setChecked(underscores);

  confWidget->addTab(smpegWidget, "SMPEG");

  vbox->addWidget(playerGroup);
  vbox->addWidget(confWidget);

#ifndef HAVE_XMMS
  xmmsButton->setEnabled(false);
#endif
#ifndef HAVE_NOATUN
  noatunButton->setEnabled(false);
#endif
#ifndef HAVE_SMPEG
  smpegButton->setEnabled(false);
#endif

}


void XmmsKdeConfigDialog::osdChanged() {

  if (!osd) {
    osd = new OSDFrame(osdSizeBox->value(), 
		       osdFGColorButton->color(),
		       osdFGColorButton->color(),
		       getOSDTransparency(),
		       OSDFrame::OSD_TEST);
  } else {
    osd->setFontSize(osdSizeBox->value());
    osd->setFGColor(osdFGColorButton->color());
    osd->setBGColor(osdBGColorButton->color(), getOSDTransparency());
  }

  osd->move(osdXBox->value(), osdYBox->value());
}


void XmmsKdeConfigDialog::showTheme(int index) {
  /*
   * load the main pixmap from the selected theme
   */
  loadTheme(tlist[index]);
}


/*
 * load a theme from a file
 *
 */
void XmmsKdeConfigDialog::loadTheme(QString themeName) {
  
  qDebug("xmms-kde: loading theme :" + themeName);

  /*
   * load theme
   */
  KSimpleConfig* themeConfig = new KSimpleConfig(themeName);
  
  themeConfig->setGroup("BITMAPS");
  QString file = themeConfig->readEntry("file");

  if (file.isNull()) {
    qDebug("xmms-kde: error loading theme");

    return;
  }

  QString tarFile = dDirs->findResource("data", "xmms-kde/" + file);
  if (tarFile.isNull()) {
    qDebug("xmms-kde: error loading theme");
    return;
  }
  
  delete themeConfig;

  qDebug("xmms-kde: trying to load skin: '" + tarFile + "'");
  QByteArray array = readFile(tarFile, "screenshot.png");

  if (array.isNull()) {
      imageLabel->setPixmap(*noScreen);
  } else {
    QPixmap *pix = new QPixmap(array);
    imageLabel->setPixmap(*pix);
  }

  theme = themeName;
}


/*
 * load a file from a tar.gz archive
 *
 */
QByteArray readFile(QString tarName, QString fileName) {

  KTarGz tar(tarName);

  if (!tar.open(IO_ReadOnly)) {
    qDebug("xmms-kde: didn't find file " + tarName);
    return 0;
  }

  const KTarDirectory* dir = tar.directory();

  const KTarEntry* e = dir->entry(fileName);
  if (e == NULL) {
    qDebug("xmms-kde: didn't find file " + fileName + " in file " + tarName);
    return 0;
  }
  
  ASSERT( e && e->isFile() );
  const KTarFile* f = (KTarFile*)e;
  
  QByteArray arr(f->data());

  tar.close();
  return arr;
}


void XmmsKdeConfigDialog::scrollModeChanged(int btn) {

  scrollMode = btn;
}

QString XmmsKdeConfigDialog::getSelectedTheme() {

  return theme;
}

PlayerInterface::Player XmmsKdeConfigDialog::getSelectedPlayer() {

  return selectedPlayer;
}

bool XmmsKdeConfigDialog::enableAccels() {

  return accelBox->isChecked();
}

int XmmsKdeConfigDialog::getScrollMode() {

  return scrollMode;
}

void XmmsKdeConfigDialog::accelButtonClicked() {

  KKeyDialog::configureKeys(accel);
}


void XmmsKdeConfigDialog::playerChanged(int index) {
  
  selectedPlayer = (PlayerInterface::Player) index;
}

/*
void XmmsKdeConfigDialog::testDB() {

  if (getEnableDB())
    db->enableDB();
  else 
    db->disableDB();

  if (db->isConnectedDB())
    db->disconnectDB();

  db->setHost(getDBHost());
  db->setPort(getDBPort());
  db->setName(getDBName());
  db->setUser(getDBUser());
  db->setPass(getDBPass());

  bool c = db->connectDB();  
}

void XmmsKdeConfigDialog::addCurrentPath() {

  pathList->insertItem(pathEdit->text());
  pathEdit->setText("");
}

void XmmsKdeConfigDialog::setCurrentPath() {

  QString dir = QFileDialog::getExistingDirectory();

  pathEdit->setText(dir);
}


void XmmsKdeConfigDialog::addSelectionToDatabase() {

  if (!db->isConnectedDB()) {
    if (!db->connectDB())
      return;
  }
  
  db->addPathToDatabase(pathList->currentText());
}

void XmmsKdeConfigDialog::setDBStatus(QString status) {

  dbStatusLabel->setText(status);
}
*/

