/*
 * configdialog.h
 */									  

#ifndef __XMMSKDECONFIG_H__
#define __XMMSKDECONFIG_H__

#include <kdialogbase.h>
#include <kglobalaccel.h>
#include <kstddirs.h>
#include <kcolorbutton.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qcolor.h>
#include <qpixmap.h>

#include "osdframe.h"
//#include "xmms-kdedb.h"

#include "playerinterface.h"

QByteArray readFile(QString tarName, QString fileName);

class XmmsKdeConfigDialog : public KDialogBase {
  Q_OBJECT

public:

  XmmsKdeConfigDialog(QWidget *parent, QStringList themeList, 
		      KStandardDirs *dataDirs, int currentScrollMode, 
		      KGlobalAccel *globalAccel,			
		      QString currentTheme,
		      PlayerInterface::Player currentPlayer,
		      bool minXmmsPlayer, bool minXmmsPlaylist,
		      bool readTags, bool underscores,
		      bool enableOsd, int osdSize, int osdx, int osdy,
		      QColor osdFGColor, QColor osdBGColor,
		      float osdTransparency
		      /*XmmsKdeDB *datab*/);
  
  virtual ~XmmsKdeConfigDialog();
  
  QString getSelectedTheme();
  
  PlayerInterface::Player getSelectedPlayer();
  
  int getScrollMode();
  
  bool enableAccels();
  
  bool getMinimizeXmmsPlayer() {
    return minimizeXmmsPlayer->isChecked();
  }
  bool getMinimizeXmmsPlaylist() {
    return minimizeXmmsPlaylist->isChecked();
  }
  
  bool getReadID3Tags() {
    return readID3Tags->isChecked();
  }
  
  bool getConvertUnderscores() {
    return convertUnderscores->isChecked();
  }
  
  QPoint getOSDPosition() {
    return QPoint(osdXBox->value(), osdYBox->value());
  }
  
  int getOSDSize() {
    return osdSizeBox->value();
  }

  float getOSDTransparency() {
    return 1 - ((float) osdTBox->value()) / 100;
  }

  
  bool getEnableOSD() {
    return osdBox->isChecked();
  }
  
  bool getEnableDB() {
    return dbBox->isChecked();
  }
  
  /*
  QString getDBHost() {
    return dbHostEdit->text();
  }
  
  int getDBPort() {
    return dbPortEdit->text().toInt();
  }
  
  QString getDBName() {
    return dbNameEdit->text();
  }
  
  QString getDBUser() {
    return dbUserEdit->text();
  }
  
  QString getDBPass() {
    return dbPassEdit->text();
  }
  */
  QColor getOSDFGColor() {
    return osdFGColorButton->color();
  }

  QColor getOSDBGColor() {
    return osdBGColorButton->color();
  }
  
protected:

private slots:
      
  void showTheme(int);

  void scrollModeChanged(int);
 
  void accelButtonClicked();
  void playerChanged(int);

  void osdChanged();

  /*
  void testDB();
  void addCurrentPath();
  void setCurrentPath();

  void addSelectionToDatabase();

  void setDBStatus(QString);
  */

private:
  void loadTheme(QString themeName);

  void setupThemesPage(QString currentTheme);

  void setupAccelPage();

  void setupOSDPage(bool enable, int size, int x, int y, QColor f, QColor b,
		    float t);

  //void setupDBPage(XmmsKdeDB *db);

  void setupPlayerPage(PlayerInterface::Player currentPlayer,
		       bool minXmmsPlayer, bool minXmmsPlaylist,
		       bool readTags, bool underscores);

  //void addPathToDatabase(XmmsKdeDB *db, QString path);

  KStandardDirs* dDirs; 
  QStringList    tlist; 
  QStringList*   nameList; 
  QListBox*      themeListBox; 
  QString        theme;
  int            scrollMode;
  QLabel*        imageLabel;
  QPixmap*       noScreen;
  KGlobalAccel*  accel;
  QCheckBox*     accelBox;
  QCheckBox*     osdBox;
  QButtonGroup*  playerGroup;

  QSpinBox *osdXBox;
  QSpinBox *osdYBox;
  QSpinBox *osdSizeBox;
  QSpinBox *osdTBox;
  KColorButton *osdFGColorButton;
  KColorButton *osdBGColorButton;
 
  QCheckBox *minimizeXmmsPlayer;
  QCheckBox *minimizeXmmsPlaylist;

  QCheckBox *readID3Tags;
  QCheckBox *convertUnderscores;

  PlayerInterface::Player selectedPlayer;

  OSDFrame *osd;
  //XmmsKdeDB *db;


  QCheckBox *dbBox;
  QLineEdit *dbHostEdit;
  QLineEdit *dbPortEdit;
  QLineEdit *dbNameEdit;
  QLineEdit *dbUserEdit;
  QLineEdit *dbPassEdit;

  QLineEdit *pathEdit;
  QListBox *pathList;

  //QLabel *dbStatusLabel;
};

#endif

