/*
 * xmms-kde.h             03.03.2001
 * 
 * 
 */

/**
 * TODO
 *   1. Color and size also for title
 *   2. Title off
 *   3. Limits for seeking and volume..
 *   5. Ability to have certain buttons on/off
 *   6. Vertical seek/volume
 *   7. Font positioning correct?
 */

#ifndef __XMMS_KDE_H__
#define __XMMS_KDE_H__

#include <qstring.h>
#include <kpanelapplet.h>
#include <kstddirs.h>
#include <qtooltip.h>
#include <qcolor.h>
#include <qfont.h>
#include <qsize.h>
#include <qpoint.h>
#include <qstrlist.h>
#include <qstringlist.h>

#include "osdframe.h"
#include "playerinterface.h"
#include "xmmsplayer.h"
#include "noatunplayer.h"
#include "smpegplayer.h"

//#include "xmms-kdedb.h"

class XmmsKdeTheme {

public:
  XmmsKdeTheme();
  ~XmmsKdeTheme();

  int loadTheme( QString themeName, KStandardDirs *dataDirs );

public: // all members that should be used in drawing.
  // remember the theme name
  QString themeName;
  
  // timers
  QColor timeColor;
  QFont  timeFont;

  QPoint timePlayedPos;
  QPoint timeTotalPos;

  // title scroller
  QFont  mainFont;
  QColor titleColor;
  QFont  titleFont;
  int    titleYPos;      
  QRect  titleClipRect;
  
  // pixmaps
  QPixmap *volumePixmap;
  QPixmap *seekPixmap;
  QPixmap *mainPixmap;

  // optional pixmaps to indicate that the user has pressed the button.
  QPixmap *backPixmap;
  QPixmap *nextPixmap;
  QPixmap *pausePixmap;
  QPixmap *stopPixmap;
  QPixmap *playPixmap;

  QPixmap *shuffleOnPixmap;
  QPixmap *repeatOnPixmap;
  
  // touch rects
  QRect rectBack, rectPlay, rectPause, rectStop, rectSkip, rectVolume, rectSeek;

  // optional touch rects
  QRect *rectShuffle, *rectRepeat;
  
  // global
  int myWidth, myHeight;

private:
  bool loadBasicSkin(QString file, QString mainBitmapFile, QString volumeBitmapFile, QString seekBitMapFile);
  bool loadExtendedSkin( QString file, QString &backBitmapFile, QString &nextBitmapFile, QString &pauseBitmapFile,
                         QString &stopBitmapFile, QString &playBitmapFile,
                         QString &shuffleBitmapFile, QString &repeatBitmapFile);
  void deleteOld();
};

class XmmsKde : public KPanelApplet, QToolTip {
  Q_OBJECT
public:
  XmmsKde(const QString& configFile, Type t = Normal, int actions = 0,
                QWidget *parent = 0, const char *name = 0);

  virtual ~XmmsKde();

  int widthForHeight(int height) const;
  int heightForWidth(int width) const;

  void about();
  void help();

  void saveConfig(); // saves the current configuration to disk!

protected:
  void paintEvent(QPaintEvent*);
  void resizeEvent(QResizeEvent*);
  void mousePressEvent(QMouseEvent*);
  void mouseReleaseEvent(QMouseEvent*);
  void mouseMoveEvent(QMouseEvent*);
  void dropEvent(QDropEvent*);
  void dragEnterEvent(QDragEnterEvent*);
  virtual void maybeTip(const QPoint&);
  virtual void wheelEvent( QWheelEvent * );
  
private:
  void setTimers(bool);
  
public slots:
  void preferences();

protected slots:


private slots:  

  void paint();
  void scroll();
  void receive();
  void loadTheme(QString);

  void previous();
  void play();
  void pause();
  void stop();
  void next();
  void volp();
  void volm();
  //void showDB();
  void exitPlayer();
  void toggleShuffle();
  void toggleRepeat();

  
 private:
  // values constantly updated from player.
  int volume;
  int trackTime;
  int currentTime;
  int currentSeekTime; // position of seek handle if in drag move!!
  int playListPos;
  bool repeatOn;
  bool shuffleOn;
  bool running;
  bool inVolumeChange;
  bool inSeekChange;  
  bool pausePressed;
  bool nextPressed;
  bool backPressed;
  bool playPressed;
  bool stopPressed;

  int titleXPosition;
  int titleScrollSpeed;
  int titleScrollMode;
  int titleWidth;
  float titleSinArg;
  
  int displayElapsedTime;  

  QTimer *scrollTimer;
  QTimer *receiveTimer;

  QString title;
  QString homeDir;
  QString resourceDir;
  KStandardDirs *dataDirs;

  QPopupMenu *themeMenu;
  QStringList *themeList;
  QStringList smpegPlayList;

  bool smpegRepeat;
  bool smpegShuffle;

  bool enableOsd;
  int osdSize;
  QPoint osdPos;
  QColor osdFGColor;
  QColor osdBGColor;
  float osdTransparency;

  //XmmsKdeDB *db;

  KConfig *mainConfig;
  KGlobalAccel  *globalAccel;
  XmmsKdeTheme theme;

  PlayerInterface *player;

  bool minimizeXmmsPlayer;
  bool minimizeXmmsPlaylist;
  bool readID3Tags;
  bool convertUnderscores;

  QSize smpegPlayListSize;
  QPoint smpegPlayListPos;

  /*
  QPoint dbQueryPos;
  QSize dbQuerySize;
  XmmsKdeDBQuery *query;
  */

  QPixmap *playListIcon;

  OSDFrame *osd;
  
};


#endif

